/*#######################################################################################
AVR DCF77 Clock 

Copyright (C) 2005 Ulrich Radig

#######################################################################################*/

#include "main.h"
#include "clock.h"

unsigned char ss	= 0;	//Globale Variable fr Sekunden
unsigned char mm	= 0;	//Globale Variable fr Minuten
unsigned char hh	= 0;	//Globale Variable fr Stunden
unsigned char day	= 0;	//Globale Variable fr den Tag
unsigned char mon	= 0;	//Globale Variable fr den Monat
unsigned int year	= 0;	//Globale Variable fr den Jahr

//Bitzhler fr RX Bit
unsigned char rx_bit_counter = 0;

//64 Bit fr DCF77 bentigt werden 59 Bits
unsigned long long dcf_rx_buffer = 0;	

//############################################################################
//Overflow Interrupt wird ausgelst bei 59Sekunde oder fehlenden DCF77 Signal 
SIGNAL (SIG_OVERFLOW1)
//############################################################################
{	
	struct  DCF77_Bits *rx_buffer;
	rx_buffer = (struct DCF77_Bits *)(unsigned long long)&dcf_rx_buffer;
		
	//Zurcksetzen des Timers
	TCNT1 = 65535 - (SYSCLK / 1024);
	//wurden alle 59 Bits empfangen?
	if (rx_bit_counter == 59)
		//Alle 59Bits empfangen stellen der Uhr nach DCF77 Buffer
		{
		//Sekunden werden auf 0 zurckgesetzt
		ss = 0;
		//Berechnung der Minuten
		mm =	1*rx_buffer->Min1+2*rx_buffer->Min2+
				4*rx_buffer->Min4+8*rx_buffer->Min8+
				10*rx_buffer->Min10+20*rx_buffer->Min20+
				40*rx_buffer->Min40;
		//Berechnung der Stunden
		hh =	1*rx_buffer->Hour1+2*rx_buffer->Hour2+
				4*rx_buffer->Hour4+8*rx_buffer->Hour8+
				10*rx_buffer->Hour10+20*rx_buffer->Hour20;
		//Berechnung des Tages
		day=	1*rx_buffer->Day1+2*rx_buffer->Day2+
				4*rx_buffer->Day4+8*rx_buffer->Day8+
				10*rx_buffer->Day10+20*rx_buffer->Day20; 
		//Berechnung des Monats
		mon=	1*rx_buffer->Month1+2*rx_buffer->Month2+
				4*rx_buffer->Month4+8*rx_buffer->Month8+
				10*rx_buffer->Month10;
		//Berechnung des Jahres
				//Berechnung der Minuten
		year=	2000+1*rx_buffer->Year1+2*rx_buffer->Year2+
				4*rx_buffer->Year4+8*rx_buffer->Year8+
				10*rx_buffer->Year10+20*rx_buffer->Year20+
				40*rx_buffer->Year40+80*rx_buffer->Year80;
		
		
		}
	else
		//nicht alle 59Bits empfangen bzw kein DCF77 Signal Uhr luft 
		//manuell weiter
		{
		ss++;//Addiere +1 zu Sekunden
		if (ss == 60)
			{
			ss = 0;
			mm++;//Addiere +1 zu Minuten
			if (mm == 60)
				{
				mm = 0;
				hh++;//Addiere +1 zu Stunden
				if (hh == 60)
					{
					hh = 0;
					}
				}
			}
		}
	//zurcksetzen des RX Bit Counters
	rx_bit_counter = 0;
	dcf_rx_buffer = 0;
};

//############################################################################
//DCF77 Modul empfngt Trger 
SIGNAL (DCF77_INT)
//############################################################################
{
	//Auswertung der Pulseweite 
	if (INT0_CONTROL == INT0_RISING_EDGE)
		{
		//Zurcksetzen des Timers
		TCNT1 = 65535 - (SYSCLK / 1024);
		//Addiere +1 zu Sekunden
		ss++;
		if (ss == 60)
			{
			ss = 0;
			}
		//Nchster Interrupt wird ausgelst bei abfallender Flanke
		INT0_CONTROL = INT0_FALLING_EDGE;
		}
	else
		{
		//Auslesen der Pulsweite von ansteigender Flanke zu abfallender Flanke
		unsigned int pulse_wide = TCNT1;
		//Zurcksetzen des Timers
		TCNT1 = 65535 - (SYSCLK / 1024);
		//berprfen ob eine 0 oder eine 1 empfangen wurde
		//0 = 100ms
		//1 = 200ms
		//Abfrage grer als 150ms
		if (pulse_wide > (65535 - (SYSCLK / 1024)/100*85))
			{
			//Schreiben einer 1 im dcf_rx_buffer an der Bitstelle rx_bit_counter
			dcf_rx_buffer = dcf_rx_buffer | ((unsigned long long) 1 << rx_bit_counter);
			}
		//Nchster Interrupt wird ausgelst bei ansteigender Flanke
		INT0_CONTROL = INT0_RISING_EDGE;
		//RX Bit Counter wird um 1 incrementiert
		rx_bit_counter++;
		}
};

//############################################################################
//Diese Routine startet und inizialisiert den Timer
void Start_Clock (void)
//############################################################################
{
	//Interrupt DCF77 einschalten auf ansteigende Flanke
	DCF77_INT_ENABLE();
	INT0_CONTROL = INT0_RISING_EDGE;
		
	//Interrupt Overfolw enable
	timer_enable_int(_BV(TOIE1));
	//Setzen des Prescaler auf 1024 
	TCCR1B |= (1<<CS10 | 0<<CS11 | 1<<CS12); 
	TCNT1 = 65535 - (SYSCLK / 1024);
	return;
};


